package com.jiuqi.grid;

import java.util.Iterator;
import java.util.Set;
import java.util.Enumeration;
import com.jiuqi.util.StreamException;
import com.jiuqi.util.inifile.StreamIni;
import com.jiuqi.util.Stream;
import java.util.Hashtable;
import java.io.Serializable;

public class KeyValues implements Serializable
{
    private static final String GRID_KEY_SIGN = "KEYS";
    private Hashtable keyMaps;
    
    public KeyValues() {
        this.keyMaps = new Hashtable();
    }
    
    public String getValue(final String key) {
        if (this.keyMaps.containsKey(key)) {
            return this.keyMaps.get(key);
        }
        return "";
    }
    
    public void setValue(final String key, final String value) {
        if (this.keyMaps.containsKey(key)) {
            this.keyMaps.remove(key);
        }
        if (value != null && value.length() > 0) {
            this.keyMaps.put(key, value);
        }
    }
    
    public boolean getAsBoolean(final String key) {
        return "1".equals(this.getValue(key));
    }
    
    public void setAsBoolean(final String key, final boolean value) {
        this.setValue(key, value ? "1" : "0");
    }
    
    public int getAsInt(final String key) {
        final String value = this.getValue(key);
        if (value == null || value.length() == 0) {
            return 0;
        }
        return Integer.parseInt(value);
    }
    
    public void setAsInt(final String key, final int value) {
        this.setValue(key, Integer.toString(value));
    }
    
    public void clear() {
        this.keyMaps.clear();
    }
    
    private void loadFromStreamIni(final Stream stream) throws StreamException {
        this.keyMaps.clear();
        if (stream == null || stream.getSize() == 0L) {
            return;
        }
        final StreamIni ini = new StreamIni(stream);
        for (int c = ini.readInteger("general", "count", 0), i = 0; i < c; ++i) {
            final String key = ini.readString("keys", Integer.toString(i), (String)null);
            final String value = ini.readString("values", Integer.toString(i), (String)null);
            if (key != null && key.length() > 0 && value != null && value.length() > 0) {
                this.keyMaps.put(key, value);
            }
        }
    }
    
    @Deprecated
    public void saveToStreamIni(final Stream stream) throws StreamException {
        stream.setSize(0L);
        final StreamIni ini = new StreamIni(stream);
        int c = 0;
        final Enumeration keys = this.keyMaps.keys();
        while (keys.hasMoreElements()) {
            final String key = keys.nextElement();
            final String value = this.keyMaps.get(key);
            ini.writeString("keys", Integer.toString(c), key);
            ini.writeString("values", Integer.toString(c), value);
            ++c;
        }
        ini.writeInteger("general", "count", c);
        ini.update();
    }
    
    public void loadFromStream(final Stream stream) throws StreamException {
        this.keyMaps.clear();
        if (stream.getSize() < 8L) {
            return;
        }
        stream.seek(-4L, 2);
        final String sign = stream.readString(4);
        if ("BINI".equals(sign)) {
            this.loadFromStreamIni(stream);
        }
        else if ("KEYS".equals(sign)) {
            stream.seek(0L, 0);
            for (int c = stream.readInt(), i = 0; i < c; ++i) {
                final String key = stream.readStringBySize();
                final String value = stream.readStringBySize();
                if (key != null && value != null) {
                    this.keyMaps.put(key, value);
                }
            }
        }
    }
    
    public void saveToStream(final Stream stream) throws StreamException {
        final Set keys = this.keyMaps.keySet();
        stream.writeInt(keys.size());
        for (final String key : keys) {
            stream.writeStringWithSize(key);
            final String value = this.keyMaps.get(key);
            stream.writeStringWithSize(value);
        }
        stream.writeString("KEYS");
    }
}
